function [LinkMatrix LinkList] = GetLink(Link)

%--------------------------------------------------------------------------
% Function to get the LinkMatrix and LinkList data structure from the Link  
%
% USAGE:      
%          [LinkMatrix LinkList] = GetLink(LinkList)
%
% INPUT:   
%               Link -- Link information structure get from the 
%                       ReadInput.m output 
% 
% OUTPUT:  
%         LinkMatrix -- (Matrix)The matrix with the link index as the elements,
%                       LinkMatrix(i,j) ~= 0 means node i and node j has
%                       a link between them, the LinkMatrix(i,j) is the 
%                       link index of that link
%           LinkList -- (Matrix)The links array 
%
% Also see ReadInput.m
%---2008--MCW--------------------------------------------------------------
%-- Ver. 1.0 -- F. Qi --

NodeNum = size(Link,1);

LinkMatrix = zeros(NodeNum, NodeNum);

for i = 1 : NodeNum
    NeighborNumber = length(Link{i,2});
    for j = 1 : NeighborNumber
        LinkMatrix(i,Link{i,2}(j)) = Link{i,3}(j);
        LinkMatrix(Link{i,2}(j),i) = Link{i,3}(j);
    end
end

LinkList = [];
for i  = 1 : NodeNum
    for j = i : NodeNum
        if (LinkMatrix(i,j) ~= 0)
            edge = sort([i j]);
            LinkList = [LinkList; edge]; %#ok<AGROW>
        end
    end
end