function MyGUIEdit(UiData, index, X, Y)

%--------------------------------------------------------------------------
% Function used to perform the drag-drop function for pattern GUI windows
%
% USAGE: 
%      MyGUIEdit(UiData, index, X, Y)
% INPUT:
%      UiData  ---  Data structure used in function 
%       index  ---  Index of the pattern in the PattInfo list
%           X  ---  X Coordinate for the point in the display windows
%           Y  ---  Y Coordinate for the point in the display windows
%      X and Y are global variables
%
% OUTPUT:
%      None
%
% Also see SubMain.m 
%
%---2008--MCW--------------------------------------------------------------
%-- Ver. 1.0 -- F. Qi --

global X Y
% Creating data-structure for the controlling line object
h = UiData.PattInfo(index).hline;
s.h_pol = h;
s.h_vert = [];
s.h_ax = get(s.h_pol,'parent');
s.h_fig = get(get(s.h_ax,'parent'),'parent');
s.controls = 'off';                                % control flag 
s.vert_index = [];                                 % index of node
s.save_x = [];                           
s.save_y = [];

s.htext = UiData.PattInfo(index).htext;
s.pair = UiData.PattInfo(index).pair;
s.link = UiData.PattInfo(index).link;
s.info = UiData.PattInfo;
s.textinfo = UiData.TextInfo;

LinkMatrix = UiData.Matrix;
set(s.h_pol,'buttondownfcn',@polygonui);
setappdata(s.h_pol,'line_data',s)    % the app data is the user defined data set. save the s information with the s.h_pol    

%% --------------------------------------------------

    function polygonui(varargin)   % Calling the edit_polygon and move_polygon
        s = getappdata(varargin{1},'line_data');
        stype = get(s.h_fig,'selectiontype');
        if ~strcmpi(stype,'open');
            return; 
        end   % check the selectiontype is open or not, if open, do nothing.  
        % check the control if is on     
        switch s.controls
            case 'on'           
                delete(s.h_vert);       
                s.h_vert = [];        % if control flag is on, then delete vrtice markers            
                s.controls = 'off';   % set it back to 'off'                        
                set(s.h_pol,'buttondownfcn',@polygonui);     % re-entry this function, like line 51            
                set(s.h_fig,'KeyPressFcn',s.KeyPressFcn_org) % link to the figure property 'KeyPressFcn'            
                setappdata(s.h_pol,'line_data',s)         % update the s variable saved in s.h_pol             
                setappdata(s.h_fig,'ui_edit_polygon_active_handle',0) % save right now active handle       
            case 'off'                       
                setappdata(s.h_fig,'ui_edit_polygon_active_handle',s.h_pol)                        
                s.controls = 'on';
                s.KeyPressFcn_org = get(s.h_fig,'KeyPressFcn');
                x = get(s.h_pol,'XData');            
                y = get(s.h_pol,'YData');
                s.h_vert = line('xdata',x,'ydata',y,'Parent',s.h_ax, 'Marker','*','color','k', 'MarkerFaceColor','none', ...
                            'linestyle','none','MarkerSize',6,'buttondownfcn',{@edit_polygon,s.h_pol}); % it is the virsul line 
                        set(s.h_pol,'buttondownfcn',{@move_polygon,s.h_pol});
                        set(s.h_fig,'KeyPressFcn',{@KeyPress_local, s.h_pol})           
                        setappdata(s.h_pol,'line_data',s)   % update the s saved in s.h_pol   
        end
    end
%% --------------------------------------------------

    function edit_polygon(obj,eventdata,h) %#ok<INUSL> calling wbm_EditPolygon and wbu_EditPolygon
        % Edit the polygon whose handle is h  
        s = getappdata(h,'line_data');    
        stype = get(s.h_fig,'selectiontype');    
        if strcmp(stype,'open')                          
            polygonui(s.h_pol,eventdata)                
            return                              
        end
        state = uisuspend_safe(s.h_fig);        % Remember initial figure state
        
        x_lim = get(s.h_ax,'xlim');             % get the axes range
        y_lim = get(s.h_ax,'ylim');
    
        current_pt = get(s.h_ax, 'CurrentPoint');
   
        % Find out which vertex is beeing edited
        x = get(s.h_pol,'XData');      
        y = get(s.h_pol,'YData');
        dif_x = x - current_pt(1,1);   
        dif_y = y - current_pt(1,2);
        dist = sqrt(dif_x.^2 + dif_y.^2);
    
        [B,IX] = sort(dist);    
        s.vert_index = IX(1);         % find which point is selected then assign the index to s.vert_index    
        s.save_x = x(s.vert_index);   % save the old x, and y value
        s.save_y = y(s.vert_index);   % needed in the "i"nsert option we do not have "i"nsert option, so we do not need this
        setappdata(h,'line_data',s);  % update the s saved with the s.h_pol
        set(s.h_fig,'WindowButtonMotionFcn',{@wbm_EditPolygon,h,[x_lim y_lim]},...
            'WindowButtonUpFcn',{@wbu_EditPolygon,h,state}, 'Pointer','fleur');   
        % 'fleur' is the normal pointer, two important callback function for motion and up. 
    end
%% --------------------------------------------------
    function wbm_EditPolygon(obj,eventdata,h,lim) %#ok<INUSL,INUSL>
        % this is the function for motion call back function, called in edit_polygon function
        s = getappdata(h,'line_data');
        pt = get(s.h_ax, 'CurrentPoint');
        % can not move out of the axes range
        if (pt(1,1) < lim(1)) || (pt(1,1) > lim(2)) || (pt(1,2) < lim(3)) || (pt(1,2) > lim(4));   
            return; 
        end
        % data point for the h, for my case, there are two points
        xx = get(s.h_pol,'XData');
        yy = get(s.h_pol,'YData');
        xx = xx(:)';                    
        yy = yy(:)';    % Make sure they are row vectors
        newx = pt(1,1);                
        newy = pt(1,2);
        
        if (s.vert_index == 1)      % If select the first point, then use the selected to substitute the first point                                    
            x = [newx xx(2)];                       
            y = [newy yy(2)];
            BeginIndex = s.pair(1);
            EndIndex = s.pair(2);
            set(s.textinfo(BeginIndex).htext,'Position',[newx newy]);
        else
            x = [xx(1) newx];       % else, use the selected point to substitute the second point
            y = [yy(1) newy];
            BeginIndex = s.pair(2);
            EndIndex = s.pair(1);
            set(s.textinfo(BeginIndex).htext,'Position',[newx newy]);
        end
        set(s.h_pol, 'XData',x, 'YData',y);
        set(s.h_vert,'XData',x, 'YData',y);
   
        % -- new part ----------------
        X(s.pair(1)) = x(1);
        X(s.pair(2)) = x(2);
        Y(s.pair(1)) = y(1);
        Y(s.pair(2)) = y(2);
        
        set(s.htext,'Position',[(X(s.pair(1)) + X(s.pair(2)))/2.0, (Y(s.pair(1)) + Y(s.pair(2)))/2.0],'String',num2str(s.link));
        a = find(LinkMatrix(BeginIndex,:));
        a = a(a ~= EndIndex);
        
        for i = 1 : length(a)
            Point1 = BeginIndex;
            Point2 = a(i);
            LinkIndex = LinkMatrix(Point1,Point2);         
            if(Point2 > Point1)         
                set(s.info(LinkIndex).hline,'XData',[X(Point1) X(Point2)],'YData',[Y(Point1) Y(Point2)]);            
                set(s.info(LinkIndex).htext,'Position',[(X(Point1) + X(Point2))/2.0, (Y(Point1) + Y(Point2))/2.0],'String',num2str(LinkIndex));        
            else
                set(s.info(LinkIndex).hline,'Xdata',[ X(Point2) X(Point1)],'YData',[Y(Point2) Y(Point1)]);
                set(s.info(LinkIndex).htext,'Position',[(X(Point1) + X(Point2))/2.0, (Y(Point1) + Y(Point2))/2.0],'String',num2str(LinkIndex));
            end
        end
        % -- end new part -----------------
        setappdata(s.h_pol,'line_data',s);
    end

%% --------------------------------------------------

    function wbu_EditPolygon(obj,eventdata,h,state) %#ok<INUSL,INUSL>
        s = getappdata(h,'line_data');
        uistack(s.h_vert,'top')      % Need to do this because the black marker has no ButtonDown and was on top
        uirestore(state);           % Restore the figure's initial state
    end

    function move_polygon(obj,eventdata,h) %#ok<INUSL,INUSL> calling wbm_MovePolygon and wbu_MovePolygon
        % Move the polygon whose handle is h, in our case it is move the line.
        s = getappdata(h,'line_data');
        stype = get(s.h_fig,'selectiontype');
        if (strcmp(stype,'open'));  
            polygonui(s.h_pol,[]);   
            return;     
        end
        state = uisuspend_safe(s.h_fig);                 % Remember initial figure state
        x_lim = get(s.h_ax,'xlim');        
        y_lim = get(s.h_ax,'ylim');    
        current_pt = get(s.h_ax, 'CurrentPoint');         % get the current position and save it as the 'old_pt' app data   
        setappdata(s.h_pol,'old_pt',[current_pt(1,1) current_pt(1,2)])
        set(s.h_fig,'WindowButtonMotionFcn',{@wbm_MovePolygon,h,[x_lim y_lim]},...
            'WindowButtonUpFcn',{@wbu_MovePolygon,h,state}, 'Pointer','fleur');
    end

%% --------------------------------------------------

    function wbm_MovePolygon(obj,eventdata,h,lim) %#ok<INUSL,INUSL>
        s = getappdata(h,'line_data');
        pt = get(s.h_ax, 'CurrentPoint');
    
        % test if the position locate the lim 
        if (pt(1,1) < lim(1)) || (pt(1,1) > lim(2)) || (pt(1,2) < lim(3)) || (pt(1,2) > lim(4));   
            return; 
        end

        % retrive the old position
        old_pt = getappdata(s.h_pol,'old_pt');
    
        % in our case xx == xv, and yy == yv, because the it is line, so only
        % two vertice, and XData and YData also two points
    
        xx = get(s.h_pol,'XData');       
        yy = get(s.h_pol,'YData');
   
        dx = pt(1,1) - old_pt(1);       dy = pt(1,2) - old_pt(2);
        xx = xx + dx;                   yy = yy + dy;
        
        setappdata(s.h_pol,'old_pt',[pt(1,1) pt(1,2)])
        set(s.h_pol, 'XData',xx, 'YData',yy);
        set(s.h_vert, 'XData',xx, 'YData',yy);
    
        % --- new part ---
        X(s.pair(1)) = xx(1); X(s.pair(2)) = xx(2);
        Y(s.pair(1)) = yy(1); Y(s.pair(2)) = yy(2);
        set(s.htext,'Position',[(X(s.pair(1)) + X(s.pair(2)))/2.0, (Y(s.pair(1)) + Y(s.pair(2)))/2.0],'String',num2str(s.link));
    
        set(s.textinfo(s.pair(1)).htext,'Position',[xx(1) yy(1)]);
        set(s.textinfo(s.pair(2)).htext,'Position',[xx(2) yy(2)]);

        a = find(LinkMatrix(s.pair(1),:));
        a = a(a ~= s.pair(2));
        for i = 1 : length(a)
            Point1 = s.pair(1);
            Point2 = a(i);
            LinkIndex = LinkMatrix(Point1,Point2);        
            if(Point2 > Point1)            
                set(s.info(LinkIndex).hline,'XData',[X(Point1) X(Point2)],'YData',[Y(Point1) Y(Point2)]);            
                set(s.info(LinkIndex).htext,'Position',[(X(Point1) + X(Point2))/2.0, (Y(Point1) + Y(Point2))/2.0],'String',num2str(LinkIndex));       
            else
                set(s.info(LinkIndex).hline,'Xdata',[X(Point2) X(Point1)],'YData',[Y(Point2) Y(Point1)]);           
                set(s.info(LinkIndex).htext,'Position',[(X(Point1) + X(Point2))/2.0, (Y(Point1) + Y(Point2))/2.0],'String',num2str(LinkIndex));        
            end
        end

        a = find(LinkMatrix(s.pair(2),:));   
        a = a(a~=s.pair(1));   
        for i = 1 : length(a)
            Point1 = s.pair(2);       
            Point2 = a(i);
            LinkIndex = LinkMatrix(Point1,Point2); 
            if(Point2 > Point1)  
                set(s.info(LinkIndex).hline,'XData',[X(Point1) X(Point2)],'YData',[Y(Point1) Y(Point2)]);     
                set(s.info(LinkIndex).htext,'Position',[(X(Point1)+ X(Point2))/2.0, (Y(Point1)+ Y(Point2))/2.0],'String',num2str(LinkIndex));   
            else
                set(s.info(LinkIndex).hline,'Xdata',[X(Point2) X(Point1)],'YData',[Y(Point2) Y(Point1)]);         
                set(s.info(LinkIndex).htext,'Position',[(X(Point1)+ X(Point2))/2.0, (Y(Point1) + Y(Point2))/2.0],'String',num2str(LinkIndex));       
            end
        end
        %--- end of new part --
        setappdata(s.h_pol,'line_data',s);
    end

%% --------------------------------------------------

    function wbu_MovePolygon(obj,eventdata,h,state) %#ok<INUSL,INUSL>
        s = getappdata(h,'line_data');
        uistack(s.h_vert,'top')     % Need to do this because the black marker has no ButtonDown and was on top
        uirestore(state);           % Restore the figure's initial state
    end
%% --------------------------------------------------

    function KeyPress_local(obj,eventdata,h)  %#ok<INUSL,INUSL>
        s = getappdata(h,'line_data');
        key = get(s.h_fig, 'CurrentCharacter');

        switch key
            case char(27)                   % escape:   stop editing
                set(s.h_fig,'selectiontype','open')
                polygonui(s.h_pol)
                return
        end
        setappdata(s.h_pol,'line_data',s)
    end

%% --------------------------------------------------

    function state = uisuspend_safe(h_fig)
        KPs = get(h_fig,'KeyPressFcn');
        try
            set(h_fig,'KeyPressFcn',KPs{1})     % Temporarly forget the line/patch handle\
        catch %#ok<CTCH>
            set(h_fig,'KeyPressFcn',KPs(1))     % Temporarly forget the line/patch handle
        end
        state = uisuspend(h_fig);           % Remember initial figure state
        state.KeyPressFcn = KPs;            % Set to its true value for use in uirestore
    end

end
