function [f] = SubMain(LinkMatrix, PatternList)
%-------------------------------------------------------------------------- 
% Sub-windows form function for displaying the KA-Patterns
%
% USAGE: 
%         [f] = SubMain(LinkMatrix, PatternList)
% 
% INPUT: 
%         LinkMatrix -- LinkMatrix of the system
%        PatternList -- All the patterns got by KA method
%
% OUTPUT: 
%                  f -- handle of the windows
%
% also see. Main.m, MyGUIEdit.m
%---2008--MCW--------------------------------------------------------------
%-- Ver. 1.1 -- F. Qi --

%% ------------ Callback Functions ---------------

% Figure resize function
    function figResize(src,evt) %#ok<INUSD,INUSD>
        fpos = get(f,'Position');
        set(LeftPanel, 'Position',...
            [0.1 0.1 fpos(3)/3-0.1 fpos(4)-0.1]);
        set(MiddlePanel, 'Position',...
            [fpos(3)/3.0+0.1 0.1 fpos(3)/3-0.1 fpos(4)-0.1]);
        set(RightPanel, 'Position',...
            [fpos(3)*2/3.0+0.1 0.1 fpos(3)/3-0.1 fpos(4)-0.1]);
    end

    function LeftPanelResize(src,evt) %#ok<INUSD,INUSD>
        bpos = get(LeftPanel,'Position');
        set(PatternBoxLabel, 'Position',...
            [7.5 bpos(4)-1.5 25 1])
        set(ListBoxPattern, 'Position',...
            [1 1 bpos(3)-2 bpos(4)-3]);
    end

    function RightPanelResize(src,evt) %#ok<INUSD,INUSD>
    end

    function MiddlePanelResize(src,evt)  %#ok<INUSD,INUSD>
    end

    function ListBoxFcn(src,evt) %#ok<INUSD,INUSD>
        vars = get(ListBoxPattern,'String');
        var_index = get(ListBoxPattern,'Value');
        NewPattern = str2num(vars(var_index,:)); %#ok<ST2NM>
        LinkMatrixNew = PatternLink(LinkMatrix, NewPattern);
        SetPattern(LinkMatrixNew, PattInfo2, TextInfo2); 
    end
%% Function to Draw patterns
    function [PattInfo TextInfo] = DrawPattern(a, Matrix)
        axes(a); %#ok<MAXES>
        set(a,'XLim',[-1.5 1.5],'YLim',[-1.5 1.5],'XTick',[],'YTick',[]);
        k = 1;
        for i = 1 : row %#ok<FXUP>
            for j = i : column
                if(Matrix(i,j)~=0)            
                    PattInfo(k).hline = line('XData',[X(i) X(j)],'YData',[Y(i) Y(j)],'Color','red','linewidth',2.5,'Marker','o','Markersize',20,'MarkerEdgeColor','red','MarkerFaceColor','red'); %#ok<AGROW>
                    PattInfo(k).htext = text((X(i)+X(j))/2.0, (Y(i)+Y(j))/2.0, num2str(LinkMatrix(i,j)),'HorizontalAlignment','Center','color','blue','fontsize',14); %#ok<AGROW>
                    PattInfo(k).pair = [i j]; %#ok<AGROW>
                    PattInfo(k).link = LinkMatrix(i,j); %#ok<AGROW>
                    k = k + 1;
                end
            end
            TextInfo(i).htext = text(X(i), Y(i),['E' num2str(i)],'HorizontalAlignment','Center','color','green'); %#ok<AGROW>
        end       
    end

%% function to generater the Pattern link matrix
    function [LinkMatrixNew] = PatternLink(LinkMatrix, NewPattern)    
        LinkInPattern = NewPattern';
        [row column] = size(LinkMatrix);
        LinkMatrixNew = LinkMatrix;
        for i = 1 : row %#ok<FXUP>
            for j = i : column
                if(~max(LinkInPattern==LinkMatrix(i,j)));
                    LinkMatrixNew(i,j) = 0;
                    LinkMatrixNew(j,i) = 0;
                end
            end
        end
    end

%% function to set patterns 
    function SetPattern(Matrix,Info2,TextInfo)
       axes(a2); %#ok<MAXES>
        k = 1;
        for i = 1 : row %#ok<FXUP>
            for j = i : column
                if (Matrix(i, j) ~= 0)
                    set(Info2(k).hline,'XData',[X(i) X(j)], 'YData',[Y(i) Y(j)]);
                    set(Info2(k).htext,'Position',[(X(i)+X(j))/2.0, (Y(i)+Y(j))/2.0],'String',num2str(LinkMatrix(i,j)));
                    k = k + 1;
                end
            end
            set(TextInfo(i).htext,'Position',[X(i) Y(i)]);
        end
    end

%% function to control pattern lines
    function ui_pattern(UiData, X, Y)        
        for i = 1 : length(UiData.PattInfo) %#ok<FXUP>
            %my_ui_edit_polygon(PattInfo, i, X, Y, LinkMatrix, TextInfo);
            MyGUIEdit(UiData, i, X, Y);
        end
    end
%% ------------ GUI layout ---------------

%% Use system background color for GUI components
panelColor = get(0,'DefaultUicontrolBackgroundColor');

%% Set up the figure and defaults
f = figure('Units','characters',...
        'Position',[30 30 120 20],...
        'Color',panelColor,...
        'HandleVisibility','callback',...
        'IntegerHandle','off',...
        'Renderer','painters',...
        'Toolbar','none',...
        'MenuBar','none',...
        'NumberTitle','off',...
        'Name','Patterns Viewer',...
        'ResizeFcn',@figResize,...
        'WindowStyle','modal');  
  
%% Create the right side panel
RightPanel = uipanel('bordertype','etchedin',...
    'BackgroundColor',panelColor,...
    'Units','characters',...
    'Position',[80.1 0.1 39.9 19.9],...
    'Parent',f,...
    'ResizeFcn',@RightPanelResize);

%% Create the Middle panel
MiddlePanel = uipanel('bordertype','etchedin',...
    'Units','characters',...
    'Position', [40.1 0.1 39.9 19.9],...
    'Parent',f,...
    'ResizeFcn',@MiddlePanelResize);

%% Creat the left panel
LeftPanel = uipanel('bordertype','etchedin',...
     'Units','characters',...
     'Position',[0.1 0.1 39.9 19.9],...
     'Parent',f,...
     'ResizeFcn',@LeftPanelResize);

%% Creat the ListBox at the leftpanel
PatternBoxLabel = uicontrol(f,'Style','text','Units','characters',...
        'Position',[7.5 18.5 25 1],...
        'String','The Patterns List',...
        'BackgroundColor',panelColor,...
        'Parent',LeftPanel);
    
ListBoxPattern = uicontrol(f,'Style','listbox','Units','characters',...
               'Position',[1 1 38 17],...
               'BackgroundColor','white',...
               'Max',1,'Min',1,...
               'FontSize',10,...
               'Parent',LeftPanel,...
               'String',num2str(PatternList),...
               'Callback',@ListBoxFcn); 
                      
%% Creat the figure at the middle panel
a1 = axes('parent',MiddlePanel,'Box','on');
set(get(a1, 'Title'),'String','Original System','Fontsize',8);
    
%% Creat the figure at the right panel
a2 = axes('parent',RightPanel,'Box','on');
set(get(a2, 'Title'),'String','Selected Pattern','Fontsize',8);

%% Dealing with the input file 
[row column] = size(LinkMatrix);

global X Y
X = zeros(row,1);
Y = zeros(column, 1);

r = 1.0;
angle = 2*pi/row;

for i = 1 : row; %#ok<FXUP>
    X(i) = r*cos(i*angle + pi/2 - angle);
    Y(i) = r*sin(i*angle + pi/2 - angle);
end

%% Drawing the link between the points (enzyme forms)
[PattInfo1 TextInfo1] = DrawPattern(a1, LinkMatrix); 

temp = PatternLink(LinkMatrix,PatternList(1,:));
[PattInfo2 TextInfo2] = DrawPattern(a2, temp);

MainUiData.PattInfo = PattInfo1;
MainUiData.TextInfo = TextInfo1;
MainUiData.Matrix = LinkMatrix;

ui_pattern(MainUiData, X, Y);

end  % whole submain end